<?php 
/*
 * AJAX Gallery: PHP Upload Handler
 * For the purpose of this demo, this handler will do the following:
 * Depending on the REQUEST_METHOD, will list & delete existing gallery items & upload new ones
 * but you'll probably want to add the gallery items to a database, maybe other processing like data validation, etc.
 * 
 * POST
 * - create album folder
 * - create full size & thumbnail folders
 * - upload the 2 versions (full & thumb) of each gallery item to their folders
 * - create a .htaccess file to force file download on images in the upload path
 * - handle errors
 * 
 * DELETE
 * - deletes gallery items
 * 
 * GET
 * - lists existing gallery items
 */

error_reporting(E_ALL | E_STRICT);

require_once 'galleryUpload/upload.class.php';

$album_id = isset($_POST['album_id']) ? $_POST['album_id'] : false;
$album_id = isset($_GET['album_id']) ? $_GET['album_id'] : $album_id;
$album_path = $album_id ? $album_id . '/' : '';

$htaccess = "ForceType application/octet-stream
<FilesMatch \"(?i)\\.(gif|jpe?g|png)$\">
\tForceType none
</FilesMatch>";

$base_path = $_SERVER['DOCUMENT_ROOT'] . GALLERY_PATH . $album_path;
if (!file_exists($base_path)) mkdir($base_path);

$upload_dir = $base_path . "files/";
if (!file_exists($upload_dir)) mkdir($upload_dir);
if (!file_exists($upload_dir . '.htaccess'))
	file_put_contents($upload_dir . '.htaccess', $htaccess);

$upload_dir_thumbnail = $base_path . "thumbnails/";
if (!file_exists($upload_dir_thumbnail)) mkdir($upload_dir_thumbnail);
if (!file_exists($upload_dir_thumbnail . '.htaccess'))
	file_put_contents($upload_dir_thumbnail . '.htaccess', $htaccess);

$upload_handler = new UploadHandler(array(
	'upload_dir' => $upload_dir,
	'upload_url' => get_base_url() . GALLERY_PATH . "{$album_path}files/",
	'image_versions' => array(
		'thumbnail' => array(
				'upload_dir' => $upload_dir_thumbnail,
				'upload_url' => get_base_url() . GALLERY_PATH . "{$album_path}thumbnails/",
				'max_width' => GALLERY_THUMB_MAX_W,
				'max_height' => GALLERY_THUMB_MAX_H
		)
	),
	'script_url' => get_base_url() . $_SERVER['SCRIPT_NAME'] . '?section=galleryUpload&album_id=' . $album_id
));

header('Pragma: no-cache');
header('Cache-Control: no-store, no-cache, must-revalidate');
header('Content-Disposition: inline; filename="files.json"');
header('X-Content-Type-Options: nosniff');
header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: OPTIONS, HEAD, GET, POST, PUT, DELETE');
header('Access-Control-Allow-Headers: X-File-Name, X-File-Type, X-File-Size');

switch ($_SERVER['REQUEST_METHOD']) {
	case 'OPTIONS':
		break;
	case 'HEAD':
	case 'GET':
		$upload_handler->get();
		break;
	case 'POST':
		if (isset($_REQUEST['_method']) && $_REQUEST['_method'] === 'DELETE') {
			$upload_handler->delete();
		} else {
			$upload_handler->post();
		}
		break;
	case 'DELETE':
		$upload_handler->delete();
		break;
	default:
		header('HTTP/1.1 405 Method Not Allowed');
}